@@ -11,7 +11,7 @@ import time
 from queue import Queue
 import json
 from swift import start_servers, SwiftElement, Button
-import phys
+from swift.phys import step_v, step_shape
 
 rtb = None
 
@@ -98,6 +98,7 @@ class Swift:
         # Element dict which holds the callback functions for form updates
         self.elements = {}
 
+        self.headless = False
         self.rendering = True
         self._notrenderperiod = 1
         self.recording = False
@@ -164,7 +165,8 @@ class Swift:
 
     def _stop_threads(self):
         self._run_thread = False
-        self.socket.join(1)
+        if not self.headless:
+            self.socket.join(1)
         if not self._dev:
             self.server.join(1)
 
@@ -195,12 +197,17 @@ class Swift:
 
         # TODO how is the pose of shapes updated prior to step?
 
+        # Update local pose of objects
         for i, obj in enumerate(self.swift_objects):
             if isinstance(obj, Shape):
                 self._step_shape(obj, dt)
             elif isinstance(obj, rtb.Robot):
                 self._step_robot(obj, dt, self.swift_options[i]["readonly"])
 
+        # Update world transform of objects
+        for obj in self.swift_objects:
+            obj._propogate_scene_tree()
+
         # Adjust sim time
         self.sim_time += dt
 
@@ -295,24 +302,24 @@ class Swift:
     #  Methods to interface with the robots created in other environemnts
     #
 
-    def add(self, ob, show_robot=True, show_collision=False, readonly=False):
+    def add(self, ob, robot_alpha=1.0, collision_alpha=0.0, readonly=False):
         """
         Add a robot to the graphical scene
 
         :param ob: the object to add
         :type ob: Robot or Shape
-        :param show_robot: Show the robot visual geometry,
-            defaults to True
-        :type show_robot: bool, optional
-        :param show_collision: Show the collision geometry,
-            defaults to False
-        :type show_collision: bool, optional
-        :return: object id within visualizer
-        :rtype: int
-        :param readonly: If true, swif twill not modify any robot attributes,
-            the robot is only nbeing displayed, not simulated,
+        :param robot_alpha: Robot visual opacity. If 0, then the geometries
+            are invisible, defaults to 1.0
+        :type robot_alpha: bool, optional
+        :param collision_alpha: Robot collision visual opacity. If 0, then
+            the geometries defaults to 0.0
+        :type collision_alpha: float, optional
+        :param readonly: If true, swift will not modify any robot attributes,
+            the robot is only being displayed, not simulated,
             defaults to False
         :type readonly: bool, optional
+        :return: object id within visualizer
+        :rtype: int
 
         ``id = env.add(robot)`` adds the ``robot`` to the graphical
             environment.
@@ -328,6 +335,8 @@ class Swift:
         # list of robots which will act upon the step() method being called.
 
         if isinstance(ob, Shape):
+            ob._propogate_scene_tree()
+            ob._added_to_swift = True
             if not self.headless:
                 id = int(self._send_socket("shape", [ob.to_dict()]))
 
@@ -355,16 +364,23 @@ class Swift:
             self._send_socket("element", ob.to_dict())
         elif isinstance(ob, rtb.ERobot):
 
-            if ob.base is None:
-                ob.base = sm.SE3()
+            # if ob.base is None:
+            #     ob.base = sm.SE3()
 
             # ob._swift_readonly = readonly
             # ob._show_robot = show_robot
             # ob._show_collision = show_collision
 
+            # Update robot transforms
+            ob._update_link_tf()
+            ob._propogate_scene_tree()
+
+            # Update robot qlim
+            ob._qlim = ob.qlim
+
             if not self.headless:
                 robob = ob._to_dict(
-                    show_robot=show_robot, show_collision=show_collision
+                    robot_alpha=robot_alpha, collision_alpha=collision_alpha
                 )
                 id = self._send_socket("shape", robob)
 
@@ -377,8 +393,8 @@ class Swift:
             self.swift_objects.append(ob)
 
             self.swift_options[int(id)] = {
-                "show_robot": show_robot,
-                "show_collision": show_collision,
+                "robot_alpha": robot_alpha,
+                "collision_alpha": collision_alpha,
                 "readonly": readonly,
             }
 
@@ -480,6 +496,21 @@ class Swift:
                 " to stop the recording"
             )
 
+    def screenshot(self, file_name="swift_snap"):
+        """
+        Save a screenshot of the current Swift frame as a png file
+
+        :param file_name: The file name for which the screenshot will be saved as
+        :type file_name: string
+
+        ``env.screenshot(file_name)`` saves a screenshot and downloads it as file_name
+        """
+
+        if file_name.endswith(".png"):
+            file_name = file_name[:-4]
+
+        self._send_socket("screenshot", [file_name])
+
     def process_events(self, events):
         """
         Process the event queue from Swift, this invokes the callback functions
@@ -492,20 +523,51 @@ class Swift:
             self.elements[event].update(events[event])
             self.elements[event].cb(events[event])
 
+    def set_camera_pose(self, position, look_at):
+        """
+        Swift.set_camera_pose(position, look_at) will set the camera
+        position and orientation of the camera within the swift scene.
+        The camera is located at location and is oriented to look at a
+        point in space defined by look_at. Note that the camera is
+        oriented with the positive z-axis.
+
+        :param position: The desired position of the camera
+        :type position: 3 vector (list or ndarray)
+        :param look_at: A point in the scene in which the camera will look at
+        :type look_at: 3 vector (list or ndarray)
+        """
+
+        # if isinstance(pose, sm.SE3):
+        #     pose = pose.A
+
+        # if look_at is None:
+        #     q = r2q(pose[:3, :3], order="xyzs").tolist()
+        # else:
+        #     q = None
+
+        if isinstance(position, np.ndarray):
+            position = position.tolist()
+
+        if isinstance(look_at, np.ndarray):
+            look_at = look_at.tolist()
+
+        transform = {
+            "t": position,
+            "look_at": look_at,
+        }
+
+        self._send_socket("camera_pose", transform, False)
+
     def _step_robot(self, robot, dt, readonly):
 
-        # robot = robot_object['ob']
+        # robot._set_link_fk(robot.q)
 
-        robot._set_link_fk(robot.q)
-
-        if readonly or robot._control_type == "p":
+        if readonly or robot._control_mode == "p":
             pass  # pragma: no cover
 
-        elif robot._control_type == "v":
+        elif robot._control_mode == "v":
 
-            phys.step_v(
-                robot._n, robot._valid_qlim, dt, robot._q, robot._qd, robot._qlim
-            )
+            step_v(robot._n, robot._valid_qlim, dt, robot._q, robot._qd, robot._qlim)
 
             # _v(robot._q, robot._qd, dt, robot._qlim, robot._valid_qlim)
 
@@ -517,18 +579,28 @@ class Swift:
             #         robot.q[i] = np.min([robot.q[i], robot.qlim[1, i]])
             #         robot.q[i] = np.max([robot.q[i], robot.qlim[0, i]])
 
-        elif robot.control_type == "a":
+        elif robot.control_mode == "a":
             pass
 
         else:  # pragma: no cover
             # Should be impossible to reach
             raise ValueError(
-                "Invalid robot.control_type. " "Must be one of 'p', 'v', or 'a'"
+                "Invalid robot.control_mode. " "Must be one of 'p', 'v', or 'a'"
             )
 
+        # Update the robot link transofrms based on the new q
+        robot._update_link_tf()
+
     def _step_shape(self, shape, dt):
 
-        phys.step_shape(dt, shape.v, shape._base, shape._sT, shape._sq)
+        if shape._changed:
+            shape._changed = False
+            id = self.swift_objects.index(shape)
+            self._send_socket("shape_update", [id, shape.to_dict()])
+
+        step_shape(
+            dt, shape.v, shape._SceneNode__T, shape._SceneNode__wT, shape._SceneNode__wq
+        )
         if shape.collision:
             shape._update_pyb()
 
@@ -565,8 +637,8 @@ class Swift:
                         [
                             i,
                             self.swift_objects[i]._fk_dict(
-                                self.swift_options[i]["show_robot"],
-                                self.swift_options[i]["show_collision"],
+                                self.swift_options[i]["robot_alpha"],
+                                self.swift_options[i]["collision_alpha"],
                             ),
                         ]
                     )
